c--julian.f
c----------------------------------------------------------------------
c  subroutines for date format changes
c    julian(year,month,day,hr,min,sec,rmjd):	
c			converts Gregorian date-time to Modified Julian Day (MJD)
c    juldat(year,month,day,hr,min,sec,rmjd):	
c			converts MJD to date-time
c    mjdfy(rmjd,fyear)	
c			converts MJD to fractional of a year
c    fydat(year,month,day,hr,min,sec,fyear)   
c			converts fractional year to date-time
c    ymdfy(year,month,day,hr,min,sec,fyear)	
c			converts date-time to fractional year
c    CALDAT(JULIAN,MM,ID,IYYY) converts 
c  functions
c    JULDAY(MM,ID,IYYY) 
c			converts (month,day,year) to modified Julian Day
c-------------------------------------------------------------------
      subroutine julian(iy,im,id,hour,min,sec,rmjd)
c
c--converts Date --> Modified Julian Days
c 
c--date is in the Gregorian Calendar
c--http://www.astro.uu.nl/~strous/AA/en/reken/juliaansedag.html
c--astronomical institute, university of utrecht
c
      implicit real*8(a-h,o-y),integer*4(i-n)
	integer*4 y,hour,sec,c
c
c--first get time in days
      d=id+((hour+(min+sec/60.d0)/60.d0)/24.d0)
	m=im
	y=iy
	if(m.lt.3) then
	  m=m+12
	  y=y-1
	endif
c
c--julian day number
      c=2-(y/100)+(y/400)
	i1=(1461*(y+4716)/4)
	i2=(153*(m+1)/5)
	rjd=int(1461*(y+4716)/4)+int(153*(m+1)/5)+d+c-1524.5
c--modified julian day
	rmjd=rjd-2400000.5d0
c
      return
	end
c--------------------------------------------------------------
      subroutine juldat(y,m,d,hour,min,sec,rmjd)
c
c--converts Modified Julian Days --> Date
c
c--date is in the Gregorian Calendar
c--http://www.astro.uu.nl/~strous/AA/en/reken/juliaansedag.html
c--astronomical institute, university of utrecht
c    (Eq. 3)  p = [J + 0.5] 
c    (Eq. 4) s1 = p + 68569 
c    (Eq. 5) n = [4*s1/146097] 
c    (Eq. 6) s2 = s1 - [(146097*n + 3)/4] 
c    (Eq. 7) i = [4000*(s2 + 1)/1461001] 
c    (Eq. 8) s3 = s2 - [1461*i/4] + 31 
c    (Eq. 9) q = [80*s3/2447] 
c    (Eq. 10) e = s3 - [2447*q/80] 
c    (Eq. 11) s4 = [q/11] 
c    (Eq. 12) m = q + 2 - 12*s4 
c    (Eq. 13) j = 100*(n - 49) + i + s4 
c    (Eq. 14) d = e + J - p + 0.5 
c
c............................................
      implicit real*8(a-h,o-y),integer*4(i-n),logical(z)
	integer*4 y,d,hour,sec,p,q,s1,s2,s3,s4
c
c--add to get full Julian day
	rjd=rmjd+2400000.5d0
c--save decimal part of day
      decday=rmjd-aint(rmjd)
c
c--start the sequence
      p = rjd+0.5d0
	s1 = p+68569
	n = (4*s1)/146097
	s2 = s1-(146097*n+3)/4
	i = 4000*(s2+1)/1461001
	s3 = s2-(1461*i)/4+31 
	q = (80*s3)/2447
	e = s3-(2447*q)/80
	s4 = q/11
	m = q+2-12*s4
	y = 100*(n-49)+i+s4
	d = e+rjd-p+0.5d0
c  
c--now decimal part as (h:m:s)
      ns1=nint(decday*86400)
	hour=ns1/3600
	ns2=ns1-hour*3600
	min=ns2/60
	sec=ns2-min*60
c
      return
	end
c-------------------------------------------------------------------
      subroutine mjdfy(rmjd,fyear)
c
c--converts modified Modified Julian Days to Fractional Year
c 
c--date is in the Gregorian Calendar
c--http://www.astro.uu.nl/~strous/AA/en/reken/juliaansedag.html
c--astronomical institute, university of utrecht
c
      implicit real*8(a-h,o-y),integer*4(i-n)
	integer*4 y,m,d,hour,min,sec
c
c--first convert rmjd to date
      call juldat(y,m,d,hour,min,sec,rmjd)
c--find julian date of start of year
      call julian(y,1,1,0,0,0,rmjd0)
c--find julian date of start of next year
      call julian(y+1,1,1,0,0,0,rmjd1)
c--compute year and fraction of year
      fyear=y+(rmjd-rmjd0)/(rmjd1-rmjd0)
c
      return
	end
c-------------------------------------------------------------------
      subroutine fydat(y,m,d,hour,min,sec,fyear)
c
c--converts Fractional year to Date
c 
c--date is in the Gregorian Calendar
c--http://www.astro.uu.nl/~strous/AA/en/reken/juliaansedag.html
c--astronomical institute, university of utrecht
c
      implicit real*8(a-h,o-y),integer*4(i-n)
	integer*4 y,m,d,hour,min,sec
c
      y=fyear
	yrem=fyear-y
c--find julian date of start of year
      call julian(y,1,1,0,0,0,rmjd0)
c--find julian date of start of next year
      call julian(y+1,1,1,0,0,0,rmjd1)
	rmjd=rmjd0+yrem*(rmjd1-rmjd0)
c--first convert rmjd to date
      call juldat(y,m,d,hour,min,sec,rmjd)
c
      return
	end
c-------------------------------------------------------------------
      subroutine ymdfy(y,m,d,hour,min,sec,fyear)
c
c--converts Date (y,m,d...) to Fractional Year
c 
c--date is in the Gregorian Calendar
c--http://www.astro.uu.nl/~strous/AA/en/reken/juliaansedag.html
c--astronomical institute, university of utrecht
c
      implicit real*8(a-h,o-y),integer*4(i-n)
	integer*4 y,m,d,hour,min,sec
c
c--first convert date to modified julian days
      call julian(y,m,d,hour,min,sec,rmjd)
c--find julian date of start of year
      call julian(y,1,1,0,0,0,rmjd0)
c--find julian date of start of next year
      call julian(y+1,1,1,0,0,0,rmjd1)
c--compute year and fraction of year
      fyear=y+(rmjd-rmjd0)/(rmjd1-rmjd0)
c
      return
	end
c-------------------------------------------------------------
      subroutine njdays(y1,m1,d1,y2,m2,d2,ndays)
c
c finds number of days between any two dates (whole days only)
c
      implicit real*8(a-h,o-z)
      integer y1,d1,y2,d2
c
      call julian(y1,m1,d1,0,0,0,rmjd1)
      call julian(y2,m2,d2,0,0,0,rmjd2)
	ndays=nint(rmjd2-rmjd1)
c
      return
	end
c-------------------------------------------------------------------
      subroutine tsamp(ii,ky,kmo,kd,kh,km,ks,delt,tunit)
c
c--finds sampling interval for first two sammples of a ggp format file
c--ii = iteration, delt is interval, tunit is unit
c
      implicit real*8(a-h,o-y),integer*4(i-n)
      character*20 tunit
c
      if(ii.gt.2) return
      if(ii.eq.1) then
		call julian(ky,kmo,kd,kh,km,ks,rd1)
	  return
      endif
c--get difference of MJD
	call julian(ky,kmo,kd,kh,km,ks,rd)
	dd=rd-rd1
c--select sampling unit and size
	if(dd.ge.1.0001d0) then
	  delt=nint(dd)
	  tunit='day'
	else
        hh=24.d0*dd
	  if(hh.ge.1.0001d0) then
	    delt=nint(hh)
	    tunit='hour'
	  else
	    rm=60.d0*hh
	    if(rm.ge.1.0001d0) then
	      delt=nint(rm)
	      tunit='min'
          else
	      ss=60.d0*rm
	      delt=nint(ss)
	      tunit='sec'
	    endif
        endif
      endif
	write(*,'(a,f8.4,a1,a10)') ' sampling ',delt,' ',tunit
      return	
c
	end
c------------------------------------------------------------------------
      subroutine ndaym(year,month,day)
c
c finds number of days in any month of year
c
      implicit real*8(a-h,o-z)
      integer year,month,day,nday(12)
      logical leapyr,lpyr
      data nday/31,28,31,30,31,30,31,31,30,31,30,31/
c--check for a leap year
      leapyr = lpyr(year)
      day=nday(month)
	if(leapyr.and.month.eq.2) day=day+1
c
      return
      end
c----------------------------------------------------------------------
      function lpyr(year)
c
c--logical function to return T, F depending whether iyear
c--is a leap year or not. 
c
      logical lpyr
      integer year
c
      lpyr=.false.
c--first rule - is it divisible by 4?
      if(mod(year,4).eq.0) lpyr=.true.
c--is year a century?
      if(mod(year,100).eq.0) then
c--it is not a leap year
        lpyr=.false.
c--unless it is also divisible by 400
        if(mod(year,400).eq.0) lpyr=.true.
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine rndjd(d2)
c
	implicit real*8(a-h),integer*4(i-n)
      data intvs/60/
	drmd=dble(intvs)/86400.d0
c
      d1=d2
      decday=d1-aint(d1)
	s1=decday*86400.d0
	i2=nint(s1/intvs)
	d2=aint(d1)+i2*drmd
	write(*,*) d1,decday,s1,i2,d2
c      pause
c
      return
	end
c-------------------------------------------------------------------------
*****************************************************************************
*     SUBROUTINE CALDAT                          NUMERICAL RECIPES V 2      *
*****************************************************************************
      SUBROUTINE CALDAT(JULIAN,MM,ID,IYYY)
      INTEGER ID,IYYY,JULIAN,MM,IGREG
      PARAMETER (IGREG=2299161)
      PARAMETER (MJDIERS=2400001)
      INTEGER JA,JALPHA,JB,JC,JD,JE
      JULIAN=JULIAN+MJDIERS
      IF(JULIAN.GE.IGREG)THEN
         JALPHA=INT(((JULIAN-1867216)-0.25)/36524.25)
         JA=JULIAN+1+JALPHA-INT(0.25*JALPHA)
      ELSE
         JA=JULIAN
      ENDIF
      JB=JA+1524
      JC=INT(6680.+((JB-2439870)-122.1)/365.25)
      JD=365*JC+INT(0.25*JC)
      JE=INT((JB-JD)/30.6001)
      ID=JB-JD-INT(30.6001*JE)
      MM=JE-1
      IF(MM.GT.12)MM=MM-12
      IYYY=JC-4715
      IF(MM.GT.2) IYYY=IYYY-1
      IF(IYYY.LE.0) IYYY=IYYY-1
      JULIAN=JULIAN-MJDIERS
      RETURN
      END
c------------------------------------------------------------------------
*****************************************************************************
*     FUNCTION JULDAY                            NUMERICAL RECIPES V 2      *
*****************************************************************************
      FUNCTION JULDAY(MM,ID,IYYY)
      INTEGER JULDAY,ID,IYYY,MM,IGREG
      PARAMETER (IGREG=15+31*(10+12*1582))
      INTEGER JA,JM,JY
      JY=IYYY
      IF (JY.EQ.0) PAUSE 'JULDAY: THERE IS NO YEAR ZERO'
      IF (JY.LT.0) JY=JY+1
      IF (MM.GT.2) THEN
         JM=MM+1
      ELSE
         JY=JY-1
         JM=MM+13
      ENDIF
      JULDAY=INT(365.25*JY)+INT(30.6001*JM)+ID+1720995
     .     -2400001
      IF (ID+31*(MM+12*IYYY).GE.IGREG) THEN
         JA=INT(0.01*JY)
         JULDAY=JULDAY+2-JA+INT(0.25*JA)
      ENDIF
      RETURN
      END
